from ..utils import AttrDict

CFM_PARAMS = AttrDict(
    {
        "sigma_min": 1e-06,
        "solver": "euler",
        "t_scheduler": "cosine",
        "training_cfg_rate": 0.2,
        "inference_cfg_rate": 0.7,
        "reg_loss_type": "l1",
    }
)
